@@ -654,7 +654,17 @@ search_status_cb (void *opaque, const char *line)
                   while (spacep (s))
                     s++;
                   if (*s)
-                    hd->last_pk_no = atoi (s);
+                    {
+                      hd->last_pk_no = atoi (s);
+                      /* Keyboxd returns 0 for invalid but also for
+                       * the primary key and 1 for the first subkey.
+                       * That does not match our use and thus we
+                       * increment it despite that this maps an
+                       * invalid index to the primary key.  */
+                      if (hd->last_pk_no >= 0)
+                        hd->last_pk_no++;
+                    }
+
                 }
             }
         }
@@ -864,6 +874,33 @@ keydb_search (KEYDB_HANDLE hd, KEYDB_SEARCH_DESC *desc,
   err = kbx_client_data_cmd (hd->kbl->kcd, line, search_status_cb, hd);
   if (!err && !(err = kbx_client_data_wait (hd->kbl->kcd, &buffer, &len)))
     {
+      int any;
+      u32 kid[2];
+
+      for (any = i = 0; i < ndesc && !any; i++)
+        if (desc[i].skipfnc)
+          any = 1;
+
+      if (any)
+        {
+          err = kbx_get_first_opgp_keyid (buffer, len, kid);
+          if (err)
+            {
+              log_info ("error getting keyid for skipping callback: %s\n",
+                        gpg_strerror (err));
+              goto leave;
+            }
+          for (i = 0; i < ndesc; i++)
+            if (desc[i].skipfnc (desc[i].skipfncvalue, kid, hd->last_uid_no))
+              { /* Callback told us to skip this one.  */
+                if (DBG_KEYDB && hd->last_ubid_valid)
+                  log_printhex (hd->last_ubid, 20, "skipped UBID (%d,%d):",
+                                hd->last_uid_no, hd->last_pk_no);
+                snprintf (line, sizeof line, "NEXT");
+                goto do_search;  /* again */
+              }
+        }
+
       hd->kbl->search_result = iobuf_temp_with_content (buffer, len);
       xfree (buffer);
       if (DBG_KEYDB && hd->last_ubid_valid)
