@@ -38,20 +38,10 @@ int pk_check_secret_key (pubkey_algo_t algo, gcry_mpi_t *skey);
 
 /*-- ecdh.c --*/
 gcry_mpi_t  pk_ecdh_default_params (unsigned int qbits);
-gpg_error_t pk_ecdh_generate_ephemeral_key (gcry_mpi_t *pkey, gcry_mpi_t *r_k);
-gpg_error_t pk_ecdh_encrypt_with_shared_point
-/*         */  (const char *shared, size_t nshared,
-                const byte pk_fp[MAX_FINGERPRINT_LEN],
-                const byte *data, size_t ndata,
-                gcry_mpi_t *pkey,
-                gcry_mpi_t *out);
-
-int pk_ecdh_encrypt (gcry_mpi_t *resarr, const byte pk_fp[MAX_FINGERPRINT_LEN],
-                     gcry_mpi_t data, gcry_mpi_t * pkey);
-int pk_ecdh_decrypt (gcry_mpi_t *result, const byte sk_fp[MAX_FINGERPRINT_LEN],
-                     gcry_mpi_t data,
-                     const byte *frame, size_t nframe,
-                     gcry_mpi_t * skey);
 
+gpg_error_t ecc_build_kdf_params (unsigned char **r_kdf_params, size_t *r_len,
+                                  const unsigned char **r_kdf_params_spec,
+                                  gcry_mpi_t *pkey,
+                                  const byte fp[MAX_FINGERPRINT_LEN]);
 
 #endif /*GNUPG_G10_PKGLUE_H*/
