@@ -31,13 +31,12 @@
 
 #include "../common/i18n.h"
 #include <gpg-error.h>
-#include "../common/exechelp.h"
 #include "../common/sysutils.h"
 #include "../common/ccparray.h"
 #include "gpgtar.h"
 
 static gpg_error_t
-check_suspicious_name (const char *name)
+check_suspicious_name (const char *name, tarinfo_t info)
 {
   size_t n;
 
@@ -47,6 +46,7 @@ check_suspicious_name (const char *name)
     {
       log_error ("filename '%s' contains a backslash - "
                  "can't extract on this system\n", name);
+      info->skipped_badname++;
       return gpg_error (GPG_ERR_INV_NAME);
     }
 #endif /*HAVE_DOSISH_SYSTEM*/
@@ -59,6 +59,7 @@ check_suspicious_name (const char *name)
     {
       log_error ("filename '%s' has suspicious parts - not extracting\n",
                  name);
+      info->skipped_suspicious++;
       return gpg_error (GPG_ERR_INV_NAME);
     }
 
@@ -66,6 +67,60 @@ check_suspicious_name (const char *name)
 }
 
 
+/* This is our version of mkdir -p.  DIRECTORY is the full filename of
+ * the directory and PREFIXLEN is the length of an intial directory
+ * part which already exists.  If STRIP is set filename is removed.
+ * If VERBOSE is set a diagnostic is printed to show the created
+ * directory.  */
+static gpg_error_t
+try_mkdir_p (const char *directory, size_t prefixlen, int strip, int verbose)
+{
+  gpg_error_t err = 0;
+  char *fname;
+  char *p;
+
+  fname = xtrystrdup (directory);
+  if (!fname)
+    return gpg_error_from_syserror ();
+
+  if (strip) /* Strip last file name. */
+    {
+      p = strrchr (fname, '/');
+      if (p)
+        *p = 0;
+    }
+  else /* Remove a possible trailing slash.  */
+    {
+      if (fname[strlen (fname)-1] == '/')
+        fname[strlen (fname)-1] = 0;
+    }
+
+  if (prefixlen >= strlen (fname))
+    goto leave; /* Nothing to create */
+
+  for (p = fname+prefixlen; (p = strchr (p, '/')); p++)
+    {
+      *p = 0;
+      err = gnupg_mkdir (fname, "-rwx------");
+      if (gpg_err_code (err) == GPG_ERR_EEXIST)
+        err = 0;
+      *p = '/';
+      if (err)
+        goto leave;
+    }
+  err = gnupg_mkdir (fname, "-rwx------");
+  if (gpg_err_code (err) == GPG_ERR_EEXIST)
+    err = 0;
+  if (!err && verbose)
+    log_info ("created   '%s/'\n", fname);
+
+ leave:
+  xfree (fname);
+  return err;
+}
+
+
+
 static gpg_error_t
 extract_regular (estream_t stream, const char *dirname,
                  tarinfo_t info, tar_header_t hdr, strlist_t exthdr)
@@ -83,7 +138,7 @@ extract_regular (estream_t stream, const char *dirname,
     if (sl->flags == 1)
       fname = sl->d;
 
-  err = check_suspicious_name (fname);
+  err = check_suspicious_name (fname, info);
   if (err)
     goto leave;
 
@@ -96,7 +151,6 @@ extract_regular (estream_t stream, const char *dirname,
     }
   fname = fname_buffer;
 
-
   if (opt.dry_run)
     outfp = es_fopen ("/dev/null", "wb");
   else
@@ -104,8 +158,18 @@ extract_regular (estream_t stream, const char *dirname,
   if (!outfp)
     {
       err = gpg_error_from_syserror ();
-      log_error ("error creating '%s': %s\n", fname, gpg_strerror (err));
-      goto leave;
+      /* On ENOENT, try afain after trying to create the directories.  */
+      if (!opt.dry_run && gpg_err_code (GPG_ERR_ENOENT)
+          && !try_mkdir_p (fname, strlen (dirname) + 1, 1, opt.verbose))
+        {
+          outfp = es_fopen (fname, "wb,sysopen");
+          err = outfp? 0 : gpg_error_from_syserror ();
+        }
+      if (err)
+        {
+          log_error ("error creating '%s': %s\n", fname, gpg_strerror (err));
+          goto leave;
+        }
     }
 
   for (n=0; n < hdr->nrecords;)
@@ -131,8 +195,12 @@ extract_regular (estream_t stream, const char *dirname,
   /* Fixme: Set permissions etc.  */
 
  leave:
-  if (!err && opt.verbose)
-    log_info ("extracted '%s'\n", fname);
+  if (!err)
+    {
+      if (opt.verbose)
+        log_info ("extracted '%s'\n", fname);
+      info->nextracted++;
+    }
   es_fclose (outfp);
   if (err && fname && outfp)
     {
@@ -146,7 +214,8 @@ extract_regular (estream_t stream, const char *dirname,
 
 
 static gpg_error_t
-extract_directory (const char *dirname, tar_header_t hdr, strlist_t exthdr)
+extract_directory (const char *dirname, tarinfo_t info,
+                   tar_header_t hdr, strlist_t exthdr)
 {
   gpg_error_t err;
   const char *name;
@@ -158,7 +227,7 @@ extract_directory (const char *dirname, tar_header_t hdr, strlist_t exthdr)
     if (sl->flags == 1)
       name = sl->d;
 
-  err = check_suspicious_name (name);
+  err = check_suspicious_name (name, info);
   if (err)
     goto leave;
 
@@ -173,38 +242,20 @@ extract_directory (const char *dirname, tar_header_t hdr, strlist_t exthdr)
   if (fname[strlen (fname)-1] == '/')
     fname[strlen (fname)-1] = 0;
 
-  if (! opt.dry_run && gnupg_mkdir (fname, "-rwx------"))
+  if (!opt.dry_run && gnupg_mkdir (fname, "-rwx------"))
     {
       err = gpg_error_from_syserror ();
+      /* Ignore existing directories while extracting.  */
       if (gpg_err_code (err) == GPG_ERR_EEXIST)
-        {
-          /* Ignore existing directories while extracting.  */
-          err = 0;
-        }
-
-      if (gpg_err_code (err) == GPG_ERR_ENOENT)
+        err = 0;
+      else if (gpg_err_code (err) == GPG_ERR_ENOENT)
         {
           /* Try to create the directory with parents but keep the
              original error code in case of a failure.  */
-          int rc = 0;
-          char *p;
-          size_t prefixlen;
-
-          /* (PREFIXLEN is the length of the new directory we use to
-           *  extract the tarball.)  */
-          prefixlen = strlen (dirname) + 1;
-
-          for (p = fname+prefixlen; (p = strchr (p, '/')); p++)
-            {
-              *p = 0;
-              rc = gnupg_mkdir (fname, "-rwx------");
-              *p = '/';
-              if (rc)
-                break;
-            }
-          if (!rc && !gnupg_mkdir (fname, "-rwx------"))
+          if (!try_mkdir_p (fname, strlen (dirname) + 1, 0, 0))
             err = 0;
         }
+
       if (err)
         log_error ("error creating directory '%s': %s\n",
                    fname, gpg_strerror (err));
@@ -228,13 +279,19 @@ extract (estream_t stream, const char *dirname, tarinfo_t info,
   if (hdr->typeflag == TF_REGULAR || hdr->typeflag == TF_UNKNOWN)
     err = extract_regular (stream, dirname, info, hdr, exthdr);
   else if (hdr->typeflag == TF_DIRECTORY)
-    err = extract_directory (dirname, hdr, exthdr);
+    err = extract_directory (dirname, info, hdr, exthdr);
   else
     {
       char record[RECORDSIZE];
 
       log_info ("unsupported file type %d for '%s' - skipped\n",
                 (int)hdr->typeflag, hdr->name);
+      if (hdr->typeflag == TF_SYMLINK)
+        info->skipped_symlinks++;
+      else if (hdr->typeflag == TF_HARDLINK)
+        info->skipped_hardlinks++;
+      else
+        info->skipped_other++;
       for (err = 0, n=0; !err && n < hdr->nrecords; n++)
         {
           err = read_record (stream, record);
@@ -324,9 +381,9 @@ gpgtar_extract (const char *filename, int decrypt)
   char *dirname = NULL;
   struct tarinfo_s tarinfo_buffer;
   tarinfo_t tarinfo = &tarinfo_buffer;
-  pid_t pid = (pid_t)(-1);
+  gpgrt_process_t proc = NULL;
   char *logfilename = NULL;
-
+  unsigned long long notextracted;
 
   memset (&tarinfo_buffer, 0, sizeof tarinfo_buffer);
 
@@ -369,19 +426,32 @@ gpgtar_extract (const char *filename, int decrypt)
     {
       strlist_t arg;
       ccparray_t ccp;
+#ifdef HAVE_W32_SYSTEM
+      HANDLE except[2] = { INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE };
+#else
+      int except[2] = { -1, -1 };
+#endif
       const char **argv;
+      gpgrt_spawn_actions_t act = NULL;
 
       ccparray_init (&ccp, 0);
       if (opt.batch)
         ccparray_put (&ccp, "--batch");
       if (opt.require_compliance)
         ccparray_put (&ccp, "--require-compliance");
-      if (opt.status_fd != -1)
+      if (opt.status_fd)
         {
           static char tmpbuf[40];
+          es_syshd_t hd;
 
-          snprintf (tmpbuf, sizeof tmpbuf, "--status-fd=%d", opt.status_fd);
+          snprintf (tmpbuf, sizeof tmpbuf, "--status-fd=%s", opt.status_fd);
           ccparray_put (&ccp, tmpbuf);
+          es_syshd (opt.status_stream, &hd);
+#ifdef HAVE_W32_SYSTEM
+          except[0] = hd.u.handle;
+#else
+          except[0] = hd.u.fd;
+#endif
         }
       if (opt.with_log)
         {
@@ -408,13 +478,27 @@ gpgtar_extract (const char *filename, int decrypt)
           goto leave;
         }
 
-      err = gnupg_spawn_process (opt.gpg_program, argv, NULL,
-                                 ((filename? 0 : GNUPG_SPAWN_KEEP_STDIN)
-                                  | GNUPG_SPAWN_KEEP_STDERR),
-                                 NULL, &stream, NULL, &pid);
+      err = gpgrt_spawn_actions_new (&act);
+      if (err)
+        {
+          xfree (argv);
+          goto leave;
+        }
+
+#ifdef HAVE_W32_SYSTEM
+      gpgrt_spawn_actions_set_inherit_handles (act, except);
+#else
+      gpgrt_spawn_actions_set_inherit_fds (act, except);
+#endif
+      err = gpgrt_process_spawn (opt.gpg_program, argv,
+                                 ((filename ? 0 : GPGRT_PROCESS_STDIN_KEEP)
+                                  | GPGRT_PROCESS_STDOUT_PIPE
+                                  | GPGRT_PROCESS_STDERR_KEEP), act, &proc);
+      gpgrt_spawn_actions_release (act);
       xfree (argv);
       if (err)
         goto leave;
+      gpgrt_process_get_streams (proc, 0, NULL, &stream, NULL);
       es_set_binary (stream);
     }
   else if (filename)
@@ -427,7 +511,7 @@ gpgtar_extract (const char *filename, int decrypt)
         {
           err = gpg_error_from_syserror ();
           log_error ("error opening '%s': %s\n", filename, gpg_strerror (err));
-          return err;
+          goto leave;
         }
       if (stream == es_stdin)
         es_set_binary (es_stdin);
@@ -442,6 +526,8 @@ gpgtar_extract (const char *filename, int decrypt)
   for (;;)
     {
       err = gpgtar_read_header (stream, tarinfo, &header, &extheader);
+      if (!err && !header)
+        break;  /* End of archive.  */
       if (err || header == NULL)
         goto leave;
 
@@ -454,32 +540,64 @@ gpgtar_extract (const char *filename, int decrypt)
       header = NULL;
     }
 
-  if (pid != (pid_t)(-1))
+  if (proc)
     {
-      int exitcode;
-
       err = es_fclose (stream);
       stream = NULL;
       if (err)
         log_error ("error closing pipe: %s\n", gpg_strerror (err));
-      else
+
+      err = gpgrt_process_wait (proc, 1);
+      if (!err)
         {
-          err = gnupg_wait_process (opt.gpg_program, pid, 1, &exitcode);
-          if (err)
+          int exitcode;
+
+          gpgrt_process_ctl (proc, GPGRT_PROCESS_GET_EXIT_ID, &exitcode);
+          if (exitcode)
             log_error ("running %s failed (exitcode=%d): %s",
                        opt.gpg_program, exitcode, gpg_strerror (err));
-          gnupg_release_process (pid);
-          pid = (pid_t)(-1);
         }
+      gpgrt_process_release (proc);
+      proc = NULL;
     }
 
-
  leave:
+  notextracted  = tarinfo->skipped_badname;
+  notextracted += tarinfo->skipped_suspicious;
+  notextracted += tarinfo->skipped_symlinks;
+  notextracted += tarinfo->skipped_hardlinks;
+  notextracted += tarinfo->skipped_other;
+  if (opt.status_stream)
+    es_fprintf (opt.status_stream, "[GNUPG:] GPGTAR_EXTRACT"
+                " %llu %llu %lu %lu %lu %lu %lu\n",
+                tarinfo->nextracted,
+                notextracted,
+                tarinfo->skipped_badname,
+                tarinfo->skipped_suspicious,
+                tarinfo->skipped_symlinks,
+                tarinfo->skipped_hardlinks,
+                tarinfo->skipped_other);
+  if (notextracted && !opt.quiet)
+    {
+      log_info ("Number of files not extracted: %llu\n", notextracted);
+      if (tarinfo->skipped_badname)
+        log_info ("     invalid name: %lu\n", tarinfo->skipped_badname);
+      if (tarinfo->skipped_suspicious)
+        log_info ("  suspicious name: %lu\n", tarinfo->skipped_suspicious);
+      if (tarinfo->skipped_symlinks)
+        log_info ("          symlink: %lu\n", tarinfo->skipped_symlinks);
+      if (tarinfo->skipped_hardlinks)
+        log_info ("         hardlink: %lu\n", tarinfo->skipped_hardlinks);
+      if (tarinfo->skipped_other)
+        log_info ("     other reason: %lu\n", tarinfo->skipped_other);
+    }
+
   free_strlist (extheader);
   xfree (header);
   xfree (dirname);
   xfree (logfilename);
   if (stream != es_stdin)
     es_fclose (stream);
+  gpgrt_process_release (proc);
   return err;
 }
