@@ -50,7 +50,7 @@ struct mime_parser_context_s
 {
   void *cookie;                /* Cookie passed to all callbacks.  */
 
-  /* The callback to announce the transation from header to body.  */
+  /* The callback to announce the transition from header to body.  */
   gpg_error_t (*t2body) (void *cookie, int level);
 
   /* The callback to announce a new part.  */
@@ -93,7 +93,7 @@ struct mime_parser_context_s
     unsigned int boundary : 1;
   } show;
 
-  struct b64state *b64state;     /* NULL or malloced Base64 decoder state.  */
+  gpgrt_b64state_t b64state;     /* NULL or malloced Base64 decoder state.  */
 
   /* A buffer for reading a mail line,  */
   char line[5000];
@@ -130,7 +130,7 @@ show_message_parser_event (rfc822parse_event_t event)
 /* Do in-place decoding of quoted-printable data of LENGTH in BUFFER.
    Returns the new length of the buffer and stores true at R_SLBRK if
    the line ended with a soft line break; false is stored if not.
-   This function asssumes that a complete line is passed in
+   This function assumes that a complete line is passed in
    buffer.  */
 static size_t
 qp_decode (char *buffer, size_t length, int *r_slbrk)
@@ -196,7 +196,7 @@ parse_message_cb (void *opaque, rfc822parse_event_t event, rfc822parse_t msg)
   const char *s;
   int rc = 0;
 
-  /* Make the RFC822 parser context availabale for callbacks.  */
+  /* Make the RFC822 parser context available for callbacks.  */
   ctx->msg = msg;
 
   if (ctx->debug)
@@ -410,17 +410,13 @@ parse_message_cb (void *opaque, rfc822parse_event_t event, rfc822parse_t msg)
                 {
                   ctx->decode_part = 2;
                   if (ctx->b64state)
-                    b64dec_finish (ctx->b64state); /* Reuse state.  */
-                  else
-                    {
-                      ctx->b64state = xtrymalloc (sizeof *ctx->b64state);
-                      if (!ctx->b64state)
-                        rc = gpg_error_from_syserror ();
-                    }
-                  if (!rc)
-                    rc = b64dec_start (ctx->b64state, NULL);
+                    gpgrt_b64dec_finish (ctx->b64state); /* Release.  */
+
+                  ctx->b64state = gpgrt_b64dec_start (NULL);
+                  if (!ctx->b64state)
+                    rc = gpg_error_from_syserror ();
                 }
-              free (value); /* Right, we need a plain free.  */
+              rfc822_free (value);
             }
         }
 
@@ -528,10 +524,7 @@ mime_parser_release (mime_parser_t ctx)
     return;
 
   if (ctx->b64state)
-    {
-      b64dec_finish (ctx->b64state);
-      xfree (ctx->b64state);
-    }
+    gpgrt_b64dec_finish (ctx->b64state);
   xfree (ctx);
 }
 
@@ -661,7 +654,7 @@ process_part_data (mime_parser_t ctx, char *line, size_t *length)
   else if (ctx->decode_part == 2)
     {
       log_assert (ctx->b64state);
-      err = b64dec_proc (ctx->b64state, line, *length, &nbytes);
+      err = gpgrt_b64dec_proc (ctx->b64state, line, *length, &nbytes);
       if (err)
         return err;
       *length = nbytes;
